// Copyright 2013 Canonical Ltd.  This software is licensed under the
// GNU Lesser General Public License version 3 (see the file COPYING).

package gwacl

import (
    "fmt"
    . "launchpad.net/gocheck"
    "net/http"
)

type TestComposeHeaders struct{}

var _ = Suite(&TestComposeHeaders{})

func (suite *TestComposeHeaders) TestNoHeaders(c *C) {
    req, err := http.NewRequest("GET", "http://example.com", nil)
    c.Assert(err, IsNil)

    observed := composeHeaders(req)
    expected := ""
    for _ = range headers_to_sign {
        expected += "\n"
    }

    c.Assert(observed, Equals, expected)
}

func (suite *TestComposeHeaders) TestFunction(c *C) {
    req, err := http.NewRequest("GET", "http://example.com", nil)
    c.Assert(err, IsNil)

    expected := ""
    for i, header_name := range headers_to_sign {
        req.Header.Set(header_name, string(i))
        expected += string(i) + "\n"
    }

    observed := composeHeaders(req)
    c.Assert(observed, Equals, expected)
}

type TestComposeCanonicalizedResource struct{}

var _ = Suite(&TestComposeCanonicalizedResource{})

func (suite *TestComposeCanonicalizedResource) TestPrependsSlash(c *C) {
}

func (suite *TestComposeCanonicalizedResource) TestFunction(c *C) {
    path := MakeRandomString(5)
    req, err := http.NewRequest("GET", fmt.Sprintf("http://example.com/%s", path), nil)
    c.Assert(err, IsNil)

    account_name := MakeRandomString(10)
    observed := composeCanonicalizedResource(req, account_name)
    expected := "/" + account_name + "/" + path
    c.Assert(observed, Equals, expected)
}
