// Copyright 2013 Canonical Ltd.  This software is licensed under the
// GNU Lesser General Public License version 3 (see the file COPYING).

package gwacl

import (
    "crypto/tls"
    "fmt"
    "net/http"
)

type X509Session struct {
    creds  Creds
    client *http.Client
}

// NewX509Session creates and returns a new X509Session based on credentials
// and X509 certificate files.
func NewX509Session(creds Creds, certFile string,
    keyFile string) (*X509Session, error) {
    cert, err := tls.LoadX509KeyPair(certFile, keyFile)
    if err != nil {
        return nil, fmt.Errorf("failed to load X509 certificate: %v", err)
    }

    cfg := &tls.Config{InsecureSkipVerify: true}
    cfg.Certificates = []tls.Certificate{cert}
    trans := &http.Transport{TLSClientConfig: cfg}
    client := &http.Client{Transport: trans}

    session := &X509Session{creds: creds, client: client}
    return session, nil
}

// NewX509SessionWithoutCert creates and returns a new X509Session
// but without any X509 transport certificates.  This is most useful in
// testing scenarios.
func NewX509SessionWithoutCert(creds Creds) (*X509Session, error) {
    client := &http.Client{}
    session := &X509Session{creds: creds, client: client}
    return session, nil
}
